/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        03.12.2008
 Description:    Commando Interpreter fr AVR_DSO

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "cmd.h"

volatile unsigned int variable[MAX_VAR];

unsigned char trigger_level = 128;
unsigned char trigger_typ = 1;
volatile unsigned int loop = 0;
unsigned char 	exec_get = 0;
unsigned char   d_trigger = 0;

COMMAND_STRUCTUR COMMAND_TABELLE[] = // Befehls-Tabelle
{
    {"GET",command_get_data},
	{"RESET",command_reset},
    {"AC",command_set_ac},
    {"DC",command_set_dc},
    {"OFFSET",command_offset},
    {"TRIGFH",command_trigger_f_high},
    {"TRIGFL",command_trigger_f_low},
    {"TRIGOFF",command_trigger_level},
    {"DIV",command_div},
    {"TRIGDEL",command_trigger_delay},
	{NULL,NULL} 
};

//------------------------------------------------------------------------------
//Commando auswerten
unsigned char extract_cmd (char *string_pointer)
{
	//Stringzeiger;
	char *string_pointer_tmp;
	unsigned char cmd_index = 0;
 
    string_pointer_tmp = strsep(&string_pointer," "); 

	//Kommando in Tabelle suchen
	while(strcasecmp(COMMAND_TABELLE[cmd_index].cmd,string_pointer_tmp))
    {
        //Abruch Whileschleife und Unterprogramm verlassen 
        if (COMMAND_TABELLE[++cmd_index].cmd == 0) return(0);
    }
    
    //Variablen finden und auswerten
	for (unsigned char a = 0; a<MAX_VAR;a++)
	{ 
        string_pointer_tmp = strsep(&string_pointer,"., ");  
		variable[a] = strtol(string_pointer_tmp,NULL,0);
	}

    //Kommando ausfhren
	COMMAND_TABELLE[cmd_index].fp();
	return(1); 
}

//------------------------------------------------------------------------------
//Reset ausfhren
void command_reset (void)
{
	RESET();
}

//------------------------------------------------------------------------------
//Eingang umschalten auf DC
void command_set_dc (void)
{
	PORTB |= (1<<PB1);
}

//------------------------------------------------------------------------------
//Eingang umschalten auf AC
void command_set_ac (void)
{
	PORTB &= ~(1<<PB1);
}

//------------------------------------------------------------------------------
//
void command_offset (void)
{
	OCR2 = variable[0];
}

//------------------------------------------------------------------------------
//
void command_div (void)
{
      switch (variable[0])
      {
        case (0):
            //Scope Time Div CPLD CLK/0
            PORTC &= ~(1<<DIV0);
            PORTC &= ~(1<<DIV1);
            break;
    
        case (1):
            //Scope Time Div CPLD CLK/2/32/256
            PORTC |= (1<<DIV0);
            PORTC &= ~(1<<DIV1);
            break;
    
        case (2):
            //Scope Time Div CPLD CLK/2/32/256
            PORTC |= (1<<DIV1);
            PORTC &= ~(1<<DIV0);
            break;
          
        case (3):
            //Scope Time Div CPLD CLK/2/32/256
            PORTC |= (1<<DIV0);
            PORTC |= (1<<DIV1);
            break;
      }
      
      loop = variable[1];
}

//------------------------------------------------------------------------------
//
void command_trigger_f_high (void)
{
	trigger_typ = 1;
}
//------------------------------------------------------------------------------
//
void command_trigger_f_low (void)
{
	trigger_typ = 0;
}
//------------------------------------------------------------------------------
//
void command_trigger_level (void)
{
	trigger_level = variable[0];
}
//------------------------------------------------------------------------------
//
void command_trigger_delay (void)
{
	d_trigger = variable[0];
}
//------------------------------------------------------------------------------
//
void command_get_data (void)
{
	exec_get = 1;
}  

void get_data (void)
{
	unsigned int trigger_counter;
 	unsigned int trigger_delay; 
    unsigned char tmp;
	unsigned char loop_tmp;
 	
    if ( exec_get)
	{

	//Trigger-Level finden
	for(trigger_counter = 0; trigger_counter < 8000;trigger_counter++)
    {  
        //RAM Adresscounter +1
        PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
        asm("nop");
        tmp = PINA;
        PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
        if (trigger_typ)
        {
            if (tmp > (trigger_level + 20))
            {
				break;
            }
        }
        else
        {
            if (tmp < (trigger_level - 20))
            {
				break;
            }
        }
    }
	trigger_counter = 8000 - trigger_counter;
    
    for(unsigned int a = 0; a < 8000 + trigger_counter;a++)
    {  
        //RAM Adresscounter +1
        PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
        asm("nop");
        tmp = PINA;
        PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
        if (trigger_typ)
        {
            if (tmp <= trigger_level)
            {
                PORTD |= (1<<PD2);
				break;
            }
        }
        else
        {
            if (tmp >= trigger_level)
            {
				PORTD |= (1<<PD2);
                break;
            }
        }
    }
	
	trigger_delay = (loop+1)*d_trigger;
    while(trigger_delay--)
    {
		PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
        PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
    }

    //Daten zum PC bertragen	
	for(unsigned int a = 0; a < 400;a++)
    {  
 		//RAM Adresscounter +1
        PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
        asm("nop");
        tmp = PINA;
        usart_write_char(tmp);
        PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
        loop_tmp = loop;  
        while(loop_tmp--)
        {
            PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
            PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
        }
    }  	
	exec_get = 0;
	}
}


